import QtQuick 2.4
import QtMultimedia 5.5

EcranPrincipalForm {

    property var urls: []

    Component.onCompleted: {
        listeVideos.append({ text: "Vidéo 1", internalId: 1})
        listeVideos.append({ text: "Vidéo 2", internalId: 2})
        listeVideos.append({ text: "Vidéo 3", internalId: 3})

        //Préparation de la liste des vidéos
        urls.push("webcam")
        urls.push("http://www.sample-videos.com/video/mp4/240/big_buck_bunny_240p_10mb.mp4")
        urls.push("http://www.sample-videos.com/video/mp4/360/big_buck_bunny_360p_10mb.mp4")
        urls.push("https://techslides.com/demos/sample-videos/small.mp4")

        //S'il n'y a pas de webcam ou de caméra branchée, on retire l'entrée de la liste
        if(QtMultimedia.defaultCamera.deviceId === "") {
            listeVideos.remove(1)
        }

        //Changement de l'état courant de la Vue
        ecranPrincipal.state = "Lecture"
    }

    comboListeVideos.onCurrentIndexChanged: {
        //Si l'utilisateur choisit un élément dans la liste
        //on récupère l'URL de la vidéo et on active le bouton de lecture
        var index = comboListeVideos.currentIndex
        var idVideo = listeVideos.get(index).internalId
        var nomVideo = listeVideos.get(index).text

        if(idVideo !== -1) {
            console.log("Vidéo sélectionnée : " + nomVideo +" (" +idVideo +")")
            btnLecture.enabled = true
            opaciteLibelle.start()
        } else {
            btnLecture.enabled = false
        }        
    }

    btnLecture.onClicked: {    
        //Si l'utilisateur déclenche la lecture...    
        ecranPrincipal.state = "Attente";
        console.log("Lecture vidéo demandée...")
        var index = comboListeVideos.currentIndex
        var idVideo = listeVideos.get(index).internalId
        var nomVideo = listeVideos.get(index).text
        
        if(idVideo !== -1) {
            //Si une vidéo a vraiment été sélectionnée,
            //on récupère son URL et on démarre la lecture.
            var url = urls[idVideo]
            console.log("URL de la vidéo : " +url)

            if(url !== "webcam") {    
                //L'affichage de la vidéo (Vue) est assuré par le Type VideoOutput
                videoOutput.source = mediaplayer
                //La lecture du flux vidéo (Modèle) est assurée par le Type MediaPlayer
                mediaplayer.source = url
                mediaplayer.play()
            } else {                      
                //Si l'utilisateur a sélectionné la webcam on définit le flux webcam
                //plutôt qu'une URL          
                videoOutput.source = camera
                ecranPrincipal.state = "Lecture"
            }
        } else {
            console.error("Aucune vidéo sélectionnée");
        }
    }

    camera.onError: {
        console.log("Erreur caméra : " +camera.errorString)
    }

    camera.onCameraStateChanged: {
        //Si la Webcam change d'état, ce slot sera exécuté
        if(camera.cameraState === Camera.UnloadedState || camera.cameraState === Camera.LoadedState) {
            ecranPrincipal.state = ""
            btnStop.enabled = false
        }
    }

    mediaplayer.onPlaying: {
        ecranPrincipal.state = "Lecture"
        btnStop.enabled = true
    }

    mediaplayer.onStopped: {
        ecranPrincipal.state = ""
        btnStop.enabled = false
    }

    btnStop.onClicked: {
        mediaplayer.stop()
        camera.stop()
    }

    //Cette animation permet de modifier l'opacité du libellé de façon progressive
    PropertyAnimation { id: opaciteLibelle; target: text1; properties: "opacity"; to: 0.0; duration: 200}

}
