import QtQuick 2.15
import QtQuick.Window 2.15
import QtWebSockets

Window {
    width: 640
    height: 480
    visible: true
    title: qsTr("Client Websocket")

    // Ce champ contient la liste des échanges entre l'utilisateur
    // et la web socket
    TextEdit {
        id: journal
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: parent.top
        anchors.bottom: chat.top
    }

    Rectangle {
        id: chat
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 10

        height: 30
        border.color: "black"

        // Ce champ de saisie de texte contiendra le texte à envoyer
        // à la web socket qui nous répondra avec une copie du texte transmis
        TextInput {
            anchors.fill: parent
            anchors.leftMargin: 10
            id: chatText
            cursorVisible: true

            verticalAlignment: Qt.AlignVCenter

            onEditingFinished: {
                //Lorsque l'utilisateur a appuyé sur la touche Entrée
                //le texte est envoyé à la web socket
                socket.sendTextMessage(chatText.text)
                //Puis on vide le champ de saisie
                chatText.clear()
            }
        }
    }

    // Déclaration de la web socket
    WebSocket {
        id: socket

        url: "wss://demo.piesocket.com/v3/channel_1?api_key=VCXCEuvhGcBDP7XhiJJUDvR1e1D3eiVjgZ9VRiaV&notify_self"
        active: true
    
        onTextMessageReceived: function (message) {
            // Si un message texte est reçu depuis la web socket
            // ce slot sera exécuté
            journal.append('   > ' + message)
        }

        onStatusChanged: {
            if (socket.status == WebSocket.Error) {
                journal.append('Erreur : ' + socket.errorString)
            } else if (socket.status == WebSocket.Open) {
                journal.append('Socket ouverte')
            } else if (socket.status == WebSocket.Closed) {
                journal.append('Socket fermée')
            }
        }
    }
}
