#include "MainWindow.h"
#include "ui_MainWindow.h"
#include <QSignalTransition>
#include <QPropertyAnimation>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    //Initialisation du timer pour le déclenchement des transitions
    //de l'automate à états "feu tricolore"
    timerFeuTricolore_.setSingleShot(false);
    timerFeuTricolore_.setInterval(2000);

    //Définition des transitions pour chaque état (état de départ, signal, état d'arrivée)
    etatFeuVert_.addTransition(&timerFeuTricolore_, SIGNAL(timeout()), &etatFeuOrange_);
    etatFeuOrange_.addTransition(&timerFeuTricolore_, SIGNAL(timeout()), &etatFeuRouge_);
    etatFeuRouge_.addTransition(&timerFeuTricolore_, SIGNAL(timeout()), &etatFeuVert_);

    //Ajout des états à l'automate à états
    automateFeux_.addState(&etatFeuVert_);
    automateFeux_.addState(&etatFeuOrange_);
    automateFeux_.addState(&etatFeuRouge_);
    automateFeux_.setInitialState(&etatFeuRouge_); //Etat initial = feu rouge

    //Définition des états graphiques -> Comment apparaissent les composants graphiquement
    etatFeuVert_.assignProperty(ui->feuTricolore, "styleSheet", "background-color: green");
    etatFeuOrange_.assignProperty(ui->feuTricolore, "styleSheet", "background-color: orange");
    etatFeuRouge_.assignProperty(ui->feuTricolore, "styleSheet", "background-color: red");

    //Définition de l'automate à états pour la voiture
    automateVoiture_.addState(&etatVoitureRoule_);
    automateVoiture_.addState(&etatVoitureSArrette_);
    automateVoiture_.addState(&etatVoitureArret_);
    automateVoiture_.setInitialState(&etatVoitureArret_);

    //Définition des états graphiques -> Comment apparaissent les composants graphiquement
    etatVoitureArret_.assignProperty(ui->label, "text", "Voiture à l'arrêt");
    etatVoitureSArrette_.assignProperty(ui->label, "text", "Voiture s'arrête");
    etatVoitureRoule_.assignProperty(ui->label, "text", "Voiture roule");

    //Définition des transitions pour chaque état (état de départ, signal, état d'arrivée)
    etatVoitureArret_.addTransition(&etatFeuRouge_, SIGNAL(exited()), &etatVoitureRoule_);
    etatVoitureRoule_.addTransition(&etatFeuOrange_, SIGNAL(entered()), &etatVoitureSArrette_);
    etatVoitureSArrette_.addTransition(&etatFeuRouge_, SIGNAL(entered()), &etatVoitureArret_);

    //Démarrage des automates et du timer qui provoquera
    //les transitions entre états
    automateFeux_.start();
    timerFeuTricolore_.start();
    automateVoiture_.start();
}

MainWindow::~MainWindow()
{
    delete ui;
}
