#include <QCoreApplication>
#include <QJsonDocument>
#include <QFile>
#include <QDebug>
#include <QJsonArray>
#include <QJsonObject>

void debugValeur(QJsonValue val) {
    if(val.type() == QJsonValue::Array) {
        qDebug() << "C'est un tableau";
        foreach(QJsonValue ligne, val.toArray()) {
            if(ligne.isString())
             qDebug() << "   -> " << ligne.toString();
            else if(ligne.isDouble())
                qDebug() << "   -> " << ligne.toDouble();
            else if(ligne.isBool())
                qDebug() << "   -> " << ligne.toBool();
            else if(ligne.isNull())
                qDebug() << "   -> null";
        }
    } else if(val.type() == QJsonValue::String) {
        qDebug() << "C'est une chaine";
        qDebug() << "   -> " << val.toString();
    }
}

int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);

    QJsonParseError error; //Cette variable recevra les erreurs de parsing
    QByteArray donnees; //Cette variable recevra les données

    //Dans un premier temps on ouvre le fichier JSON
    QFile fichier(":/fichier.json") ;
    if(fichier.open(QFile::ReadOnly)) {
       donnees = fichier.readAll(); //On lit tout le contenu du fichier JSON

       //On parse directement les données, les erreurs iront dans la variable
       //error le cas échéant. Le résultat est un document JSON.
       QJsonDocument doc = QJsonDocument::fromJson(donnees, &error);

       //On vérifie l'absence d'erreur
       if(error.error != QJsonParseError::NoError) {
           qCritical() << "Impossible d'interpréter le fichier : " << error.errorString();
       } else {
           qDebug() << "Fichier interprété";

           //On interprête le document JSON comme tableau
           QJsonArray array = doc.array();
           qDebug() << array.at(1).toString(); //On lit le premier élément

           //On interprête le document JSON comme un objet
           QJsonObject obj = doc.object();

           //On lit les valeurs des éléments
           qDebug() << "\rPremière méthode";
           foreach(QJsonValue val, obj) {
               debugValeur(val);
           }

           //On lit uniquement les clés des éléments
           qDebug() << "\rSeconde méthode";
           foreach(QString cle, obj.keys()) {
               qDebug() << "clé : " << cle;
               debugValeur(obj[cle]);
           }
       }
    }


    qDebug() << "\rEcriture d'un fichier JSON";
    QJsonObject racine; //Il y a toujours un object racine dans un document JSON
    QJsonArray tableau;
    tableau << 1 << 2 << "trois" << "quatre" << true << QJsonValue();
    racine.insert("Tableau", tableau); //Création d'une liste de valeurs
    racine.insert("Clé", "Valeur"); //Création d'une clé: valeur texte
    racine.insert("Japonais", "日本語"); //Création d'une clé: valeur texte

    QJsonDocument doc2(racine); //Création d'un second document JSON
    //Nous allons afficher dans la log le contenu sérialisé du document JSON
    //que nous venons de créer, normalement ce contenu devrait aller dans un fichier.
    qDebug() << QString::fromUtf8(doc2.toJson(QJsonDocument::Compact));

}
