#include <QStringList>
#include <QDebug>

#include "Controleur.h"

Controleur::Controleur(QObject *parent) :
    QObject(parent), process_(new QProcess(this))
{
}

void Controleur::demarrePing()
{
    //Préparation de la liste des arguments pour la commande
    QStringList args;
    args << "127.0.0.1";

    //Connexion aux différents signaux permettant de suivre
    //le déroulement de l'exécution
    connect(process_, SIGNAL(started()), this, SLOT(onStarted()));
    connect(process_, SIGNAL(finished(int)), this, SLOT(onFinished(int)));
    connect(process_, SIGNAL(readyReadStandardOutput()), this, SLOT(onDataReceived()));
    connect(process_, SIGNAL(error(QProcess::ProcessError)), this, SLOT(onError(QProcess::ProcessError)));

    //Démarrage du processus externe avec les arguments
    process_->start("ping", args);
}

void Controleur::onDataReceived()
{
    //Lorsque des données sont écrites par le processus sur
    //la sortie standard, elles arrivent dans ce slot
    qDebug() << process_->readAllStandardOutput();
}

void Controleur::onError(QProcess::ProcessError error)
{
    //Si une erreur se produit, ce slot sera exécuté
    qDebug() << "Une erreur s'est produite : " << error;
}

void Controleur::onFinished(int returnCode)
{
    //A la fin de l'exécution, ce slot est exécuté
    qDebug() << "Ping terminé avec le code " << returnCode;
}

void Controleur::onStarted()
{
    //Si le processus est correctement démarré, ce slot est appelé
    qDebug() << "Ping démarré";
}
