#include "EcranClient.h"
#include "ui_EcranClient.h"
#include "Client.h"
#include <QSharedPointer>
#include "Constants.h"

EcranClient::EcranClient(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::EcranClient)
    , remoteNode_(new QRemoteObjectNode(this))
{
    ui->setupUi(this);

    //Si l'on souhaite communiquer en direct entre le client
    //et le serveur on se passera de REGISTRY
#ifdef AVEC_REGISTRY
    remoteNode_->connectToNode(URL_REGISTRE);
#else
    remoteNode_->connectToNode(URL_SOURCE);
#endif

    //Création d'un pointeur partagé permettant d'inverser
    //le contrôle sur le serveur
    QSharedPointer<SliderReplica> ptr;
    ptr.reset(remoteNode_->acquire<SliderReplica>());

    //Instanciation du client
    client_ = new Client(ptr, this);

    //Si le client reçoit une valeur, on l'affichera
    connect(client_, &Client::echo, this, &EcranClient::onEcho);
}

EcranClient::~EcranClient()
{
    delete ui;
}


void EcranClient::onEcho(int valeur)
{
    //Si le client reçoit une valeur, ce slot sera appelé
    qDebug() << "Valeur reçue en écho : " << valeur;
    ui->valeurRecue->setText(QString::number(valeur));
}
